@@ -2,7 +2,7 @@
 # SPDX-License-Identifier: GPL-3.0-or-later
 import os
 from pathlib import Path
-from pmb.types import AportGenEntry, PathString
+from pmb.types import PathString
 import sys
 from collections.abc import Sequence
 
@@ -11,11 +11,14 @@ from collections.abc import Sequence
 #
 # FIXME (#2324): this sucks, we should re-organise this and not rely on "lifting"
 # this functions this way
-from pmb.config.file import load, save, serialize
+from pmb.config.file import load as load, save as save, serialize as serialize
 from pmb.config.sudo import which_sudo
-from pmb.config.other import is_systemd_selected
-from .init import require_programs
-from . import workdir
+from pmb.config.other import is_systemd_selected as is_systemd_selected
+from .init import (
+    require_programs as require_programs,
+    ask_for_mainline_downstream as ask_for_mainline_downstream,
+)
+from . import workdir as workdir
 
 
 #
@@ -30,8 +33,10 @@ apk_keys_path: Path = pmb_src / "pmb/data/keys"
 # (which may contain a vulnerable apk/openssl, and allows an attacker to
 # exploit the system!)
 apk_tools_min_version = {
-    "edge": "2.14.6-r2",
-    "v3.21": "2.14.6-r2",
+    "edge": "3.0.3-r1",
+    "v3.23": "3.0.3-r1",
+    "v3.22": "2.14.9-r3",
+    "v3.21": "2.14.6-r3",
     "v3.20": "2.14.4-r1",
     "v3.19": "2.14.4-r0",
     "v3.18": "2.14.4-r0",
@@ -104,13 +109,14 @@ styles = {
     "BLUE": "\033[94m",
     "BOLD": "\033[1m",
     "GREEN": "\033[92m",
+    "MAGENTA": "\033[95m",
     "RED": "\033[91m",
     "YELLOW": "\033[93m",
     "END": "\033[0m",
 }
 
 if "NO_COLOR" in os.environ:
-    for style in styles.keys():
+    for style in styles:
         styles[style] = ""
 
 # Supported filesystems and their fstools packages
@@ -158,6 +164,7 @@ host_path = os.environ["PATH"] + ":/usr/sbin/"
 # Use no more than one dir after /mnt/pmbootstrap, see remove_mnt_pmbootstrap.
 chroot_mount_bind = {
     "/proc": "/proc",
+    "/sys": "/sys",
     "$WORK/cache_apk_$ARCH": "/var/cache/apk",
     "$WORK/cache_appstream/$ARCH/$CHANNEL": "/mnt/appstream-data",
     "$WORK/cache_ccache_$ARCH": "/mnt/pmbootstrap/ccache",
@@ -204,6 +211,8 @@ chroot_device_nodes = [
     [666, "c", 1, 7, "full"],
     [644, "c", 1, 8, "random"],
     [644, "c", 1, 9, "urandom"],
+    [666, "c", 5, 0, "tty"],
+    [666, "c", 5, 2, "ptmx"],
 ]
 
 # Age in hours that we keep the APKINDEXes before downloading them again.
@@ -218,11 +227,7 @@ chroot_outdated = 3600 * 24 * 2
 # for the first time
 # IMPORTANT: the order here matters, it is the order these packages will
 # be built in (if needed). abuild must be first!
-#
-# NOTE: full hexdump is installed to workaround a bug in busybox,
-# see https://gitlab.postmarketos.org/postmarketOS/pmaports/-/issues/3268. This can be
-# reverted when the bug is properly fixed.
-build_packages = ["abuild", "apk-tools", "build-base", "ccache", "git", "hexdump"]
+build_packages = ["abuild", "apk-tools", "build-base", "ccache", "git", "pigz"]
 
 #
 # PARSE
@@ -252,11 +257,15 @@ apkbuild_package_attributes = {
     # postmarketos-base and UI meta-packages can define the default package
     # to select during "_pmb_select".
     "_pmb_default": {"array": True},
+    # kernel packages can specify one or more defconfig to use when generating
+    # the full config
+    "_defconfig": {"array": True},
 }
 
 # Variables in APKBUILD files that get parsed
 apkbuild_attributes = {
     **apkbuild_package_attributes,
+    "maintainer": {},
     "arch": {"array": True},
     "depends_dev": {"array": True},
     "makedepends": {"array": True},
@@ -284,6 +293,7 @@ apkbuild_attributes = {
     "_llvmver": {},
     # Overridden packages
     "_pkgver": {},
+    "_pkgrel": {},
     "_pkgname": {},
     # git commit
     "_commit": {},
@@ -301,7 +311,9 @@ apkbuild_custom_valid_options = [
     "!pmb:crossdirect",
     "!pmb:kconfigcheck",
     "pmb:cross-native",
-    "pmb:gpu-accel",
+    "pmb:cross-native2",
+    "pmb:drm",
+    "pmb:gpu-accel",  # deprecated
     "pmb:strict",
     "pmb:systemd",
     "pmb:systemd-never",
@@ -371,6 +383,7 @@ $IMAGE: Path to the combined boot/rootfs image
 $IMAGE_SPLIT_BOOT: Path to the (split) boot image
 $IMAGE_SPLIT_ROOT: Path to the (split) rootfs image
 $PARTITION_KERNEL: Partition to flash the kernel/boot.img to
+$PARTITION_VENDOR_BOOT: Partition to flash the vendor_boot.img to
 $PARTITION_ROOTFS: Partition to flash the rootfs to
 
 Fastboot specific: $KERNEL_CMDLINE
@@ -384,6 +397,9 @@ flashers: dict[str, dict[str, bool | list[str] | dict[str, list[list[str]]]]] =
             "list_devices": [["fastboot", "devices", "-l"]],
             "flash_rootfs": [["fastboot", "flash", "$PARTITION_ROOTFS", "$IMAGE"]],
             "flash_kernel": [["fastboot", "flash", "$PARTITION_KERNEL", "$BOOT/boot.img$FLAVOR"]],
+            "flash_vendorboot": [
+                ["fastboot", "flash", "$PARTITION_VENDOR_BOOT", "$BOOT/vendor_boot.img"]
+            ],
             "flash_vbmeta": [
                 # Generate vbmeta image with "disable verification" flag
                 [
@@ -401,6 +417,11 @@ flashers: dict[str, dict[str, bool | list[str] | dict[str, list[list[str]]]]] =
             ],
             "flash_dtbo": [["fastboot", "flash", "$PARTITION_DTBO", "$BOOT/dtbo.img"]],
             "boot": [["fastboot", "--cmdline", "$KERNEL_CMDLINE", "boot", "$BOOT/boot.img$FLAVOR"]],
+            # Like boot, but stage vendor_boot first
+            "boot_gki": [
+                ["fastboot", "stage", "$BOOT/vendor_boot.img"],
+                ["fastboot", "boot", "$BOOT/boot.img$FLAVOR"],
+            ],
             "flash_lk2nd": [["fastboot", "flash", "$PARTITION_KERNEL", "$BOOT/lk2nd.img"]],
         },
     },
@@ -418,6 +439,7 @@ flashers: dict[str, dict[str, bool | list[str] | dict[str, list[list[str]]]]] =
             "flash_rootfs": [["fastboot", "flash", "$PARTITION_ROOTFS", "$IMAGE_SPLIT_ROOT"]],
             "flash_kernel": [["fastboot", "flash", "$PARTITION_KERNEL", "$IMAGE_SPLIT_BOOT"]],
             "flash_boot": [["fastboot", "flash", "boot", "$BOOT/boot.img$FLAVOR"]],
+            "boot": [["fastboot", "--cmdline", "$KERNEL_CMDLINE", "boot", "$BOOT/boot.img$FLAVOR"]],
         },
     },
     # Some Samsung devices need the initramfs to be baked into the kernel (e.g.
@@ -584,16 +606,6 @@ git_repos = {
 #
 # APORTGEN
 #
-aportgen: dict[str, AportGenEntry] = {
-    "cross": {
-        "prefixes": ["busybox-static", "gcc", "musl", "grub-efi"],
-        "confirm_overwrite": False,
-    },
-    "device/testing": {
-        "prefixes": ["device", "linux"],
-        "confirm_overwrite": True,
-    },
-}
 
 # Use a deterministic mirror URL instead of CDN for aportgen. Otherwise we may
 # generate a pmaport that wraps an apk from Alpine (e.g. musl-armv7) locally
@@ -611,7 +623,7 @@ aportgen_mirror_alpine = "http://dl-4.alpinelinux.org/alpine/"
 newapkbuild_arguments_strings = [
     ["-n", "pkgname", "set package name (only use with SRCURL)"],
     ["-d", "pkgdesc", "set package description"],
-    ["-l", "license", "set package license identifier from" " <https://spdx.org/licenses/>"],
+    ["-l", "license", "set package license identifier from <https://spdx.org/licenses/>"],
     ["-u", "url", "set package URL"],
 ]
 newapkbuild_arguments_switches_pkgtypes = [
@@ -648,4 +660,4 @@ upgrade_ignore = [
 # CI
 #
 # Valid options for 'pmbootstrap ci', see https://postmarketos.org/pmb-ci
-ci_valid_options = ["native", "slow"]
+ci_valid_options = ["native", "slow", "with-dot-git"]
