@@ -20,16 +20,15 @@ from typing import Any
 import pmb.aportgen
 import pmb.config
 import pmb.config.pmaports
+from pmb.config.pmaports import DEVELOPMENT_CHANNEL
 from pmb.core import Config
 from pmb.types import Apkbuild, PmbArgs
 import pmb.helpers.cli
 import pmb.helpers.devices
 import pmb.helpers.git
 import pmb.helpers.http
-import pmb.helpers.logging
 import pmb.helpers.other
 import pmb.helpers.pmaports
-import pmb.helpers.run
 import pmb.helpers.ui
 import pmb.chroot.zap
 import pmb.parse.deviceinfo
@@ -357,7 +356,15 @@ def ask_for_provider_select(apkbuild: dict[str, Any], providers_cfg: dict[str, s
     """
     for select in apkbuild["_pmb_select"]:
         providers = pmb.helpers.pmaports.find_providers(select, apkbuild["_pmb_default"])
-        logging.info(f"Available providers for {select} ({len(providers)}):")
+        provider_count = len(providers)
+
+        if provider_count == 0:
+            logging.warning(
+                f"WARNING: Skipping provider selection for {select}, no providers found! This is probably a packaging bug."
+            )
+            continue
+
+        logging.info(f"Available providers for {select} ({provider_count}):")
 
         has_default = False
         providers_short = {}
@@ -488,7 +495,23 @@ def ask_for_mainline_downstream() -> pmb.helpers.devices.DeviceCategory:
             raise ValueError(f"Unexpected port_type {port_type}")
 
 
-def ask_for_device(context: Context) -> tuple[str, bool, str]:
+def switch_channel(config: Config, channel: str) -> None:
+    """Wrapper around `pmb.config.pmaports.switch_to_channel_branch(channel)` that also handles
+    installing git hooks if master is selected."""
+    pmb.config.pmaports.switch_to_channel_branch(channel)
+    # FIXME: ???
+    config.is_default_channel = False
+
+    # Copy the git hooks if master was checked out. (Don't symlink them and
+    # only do it on master, so the git hooks don't change unexpectedly when
+    # having a random branch checked out.)
+    branch_current = pmb.helpers.git.rev_parse(pkgrepo_default_path(), extra_args=["--abbrev-ref"])
+    if branch_current == "master":
+        logging.info("NOTE: pmaports is on master branch, copying git hooks.")
+        pmb.config.pmaports.install_githooks()
+
+
+def ask_for_device(context: Context, channel: str) -> tuple[str, bool, str]:
     """
     Prompt for the device vendor, model, and kernel.
 
@@ -497,6 +520,21 @@ def ask_for_device(context: Context) -> tuple[str, bool, str]:
         * device_exists: bool indicating if device port exists in repo
         * kernel: type of kernel (downstream, etc)
     """
+
+    styles = pmb.config.styles
+
+    def new_port_message(has_codename: bool = True) -> str:
+        """Return the message for new ports."""
+        return (
+            "If you're trying to select a device that is supported, check the following:\n\n"
+            f"* Make sure you spelled the vendor name{' and codename' if has_codename else ''} correctly. \n"
+            "* Check if you're on the right release branch. Devices in the 'downstream' category are only"
+            " available on the 'edge' branch; if you selected a stable branch, run pmbootstrap init again"
+            " and select 'edge'.\n"
+            "* If the device is supported by a generic port, type in the vendor and codename of the generic port.\n\n"
+            "If you want to create a new device port, follow the guide at <https://postmarketos.org/porting/>."
+        )
+
     vendors = sorted(pmb.helpers.devices.list_vendors())
     logging.info(
         "Choose your target device vendor (either an existing one, or a new one for porting)."
@@ -516,14 +554,15 @@ def ask_for_device(context: Context) -> tuple[str, bool, str]:
         codenames = []
         if new_vendor:
             logging.info(
-                f"The specified vendor ({vendor}) could not be found in existing ports, do you want"
-                " to start a new port?"
+                f"{styles['BOLD']}The specified vendor ({vendor}) could not be found in existing ports.{styles['END']}\n"
+                + new_port_message(has_codename=False)
             )
-            if not pmb.helpers.cli.confirm(default=True):
+            if not pmb.helpers.cli.confirm(
+                "Do you want to create a new device port?", default=True
+            ):
                 continue
         else:
             device_list = "Devices are categorised as follows, from best to worst:\n"
-            styles = pmb.config.styles
             for category in pmb.helpers.devices.DeviceCategory.shown():
                 device_list += f"* {category.color()}{str(category).capitalize()}{styles['END']}: {category.explain()}.\n"
             device_entries = pmb.helpers.devices.list_codenames(vendor)
@@ -553,14 +592,29 @@ def ask_for_device(context: Context) -> tuple[str, bool, str]:
             device_category = pmb.helpers.devices.get_device_category_by_apkbuild_path(device_path)
         if device_path is None:
             if device == context.config.device:
-                raise RuntimeError(
+                raise NonBugError(
                     "This device does not exist anymore, check <https://postmarketos.org/renamed>"
                     " to see if it was renamed"
                 )
-            logging.info(f"You are about to do a new device port for '{device}'.")
-            if not pmb.helpers.cli.confirm(default=True):
+
+            channel_is_improper = channel != DEVELOPMENT_CHANNEL
+
+            logging.info(
+                f"{styles['BOLD']}The specified device ('{device}') could not be found in existing ports.{styles['END']}\n"
+                + new_port_message(has_codename=True)
+            )
+            if channel_is_improper:
+                logging.info(
+                    f"Earlier you selected the '{channel}' channel. All new ports and packages must "
+                    f"go through '{DEVELOPMENT_CHANNEL}', so it will be switched to if you confirm."
+                )
+            if not pmb.helpers.cli.confirm(
+                "Do you want to create a new device port?", default=True
+            ):
                 current_vendor = vendor
                 continue
+            if channel_is_improper:
+                switch_channel(context.config, DEVELOPMENT_CHANNEL)
 
             device_category = ask_for_mainline_downstream()
 
@@ -675,10 +729,8 @@ def ask_for_mirror() -> str:
     logging.info("list of available mirrors:")
     mirrors = json.loads(s)
     keys = mirrors.keys()
-    i = 1
-    for key in keys:
+    for i, key in enumerate(keys, start=1):
         logging.info(f"[{i}]\t{key} ({mirrors[key]['location']})")
-        i += 1
     logging.info("choose 'best' to select the one closest to you")
 
     urls = []
@@ -893,20 +945,10 @@ def frontend(args: PmbArgs) -> None:
 
     # Choose release channel, possibly switch pmaports branch
     channel = ask_for_channel(config)
-    pmb.config.pmaports.switch_to_channel_branch(channel)
-    # FIXME: ???
-    config.is_default_channel = False
-
-    # Copy the git hooks if master was checked out. (Don't symlink them and
-    # only do it on master, so the git hooks don't change unexpectedly when
-    # having a random branch checked out.)
-    branch_current = pmb.helpers.git.rev_parse(pkgrepo_default_path(), extra_args=["--abbrev-ref"])
-    if branch_current == "master":
-        logging.info("NOTE: pmaports is on master branch, copying git hooks.")
-        pmb.config.pmaports.install_githooks()
+    switch_channel(config, channel)
 
     # Device
-    device, device_exists, kernel = ask_for_device(get_context())
+    device, device_exists, kernel = ask_for_device(get_context(), channel)
     config.device = device
     config.kernel = kernel
 
@@ -923,6 +965,7 @@ def frontend(args: PmbArgs) -> None:
     config.user = ask_for_username(config.user)
     ask_for_provider_select_pkg("postmarketos-base", config.providers)
     ask_for_provider_select_pkg("postmarketos-base-ui", config.providers)
+    ask_for_provider_select_pkg("postmarketos-usb-moded", config.providers)
 
     # UI and various build options
     ui = ask_for_ui(deviceinfo)
