@@ -9,24 +9,24 @@ from pmb.core.pkgrepo import (
     pkgrepo_relative_path,
 )
 from pmb.helpers import logging
+from pmb.helpers.exceptions import NonBugError
 import os
 import sys
-from typing import cast, Any
+from typing import Final
 
 import pmb.config
 from pmb.meta import Cache
 import pmb.helpers.git
 import pmb.helpers.pmaports
 import pmb.parse.version
+from pmb.types import RunOutputTypeDefault
 
 
-def clone() -> None:
-    logging.info(
-        "Setting up the native chroot and cloning the package build" " recipes (pmaports)..."
-    )
+def clone(do_shallow: bool = False) -> None:
+    logging.info("Setting up the native chroot and cloning the package build recipes (pmaports)...")
 
     # Set up the native chroot and clone pmaports
-    pmb.helpers.git.clone("pmaports")
+    pmb.helpers.git.clone("pmaports", do_shallow)
 
 
 def check_version_pmaports(real: str) -> None:
@@ -55,7 +55,7 @@ def check_version_pmbootstrap(min_ver: str) -> None:
 
     # Show versions
     logging.info(
-        f"NOTE: you are using pmbootstrap version {real}, but" f" version {min_ver} is required."
+        f"NOTE: you are using pmbootstrap version {real}, but version {min_ver} is required."
     )
 
     # Error for git clone
@@ -84,7 +84,7 @@ def read_config_repos() -> dict[str, configparser.SectionProxy]:
     cfg.read(f"{pkgrepo_default_path()}/pmaports.cfg")
 
     ret = {}
-    for section in cfg.keys():
+    for section in cfg:
         if not section.startswith("repo:"):
             continue
         repo = section.split("repo:", 1)[1]
@@ -93,8 +93,10 @@ def read_config_repos() -> dict[str, configparser.SectionProxy]:
     return ret
 
 
-@Cache("aports")
-def read_config(aports: Path | None = None) -> dict[str, Any]:
+@Cache("aports", "add_systemd_prefix")
+def read_config(
+    aports: Path | None = None, add_systemd_prefix: bool = True
+) -> configparser.SectionProxy:
     """Read and verify pmaports.cfg. If aports is not
     specified and systemd is enabled, the returned channel
     will be the systemd one (e.g. systemd-edge instead of edge)
@@ -117,7 +119,7 @@ def read_config(aports: Path | None = None) -> dict[str, Any]:
     # Require the config
     path_cfg = aports / "pmaports.cfg"
     if not os.path.exists(path_cfg):
-        raise RuntimeError("Invalid pmaports repository, could not find the" f" config: {path_cfg}")
+        raise RuntimeError(f"Invalid pmaports repository, could not find the config: {path_cfg}")
 
     # Load the config
     cfg = configparser.ConfigParser()
@@ -131,20 +133,17 @@ def read_config(aports: Path | None = None) -> dict[str, Any]:
     # Translate legacy channel names
     ret["channel"] = pmb.helpers.pmaports.get_channel_new(ret["channel"])
 
-    if systemd:
+    if systemd and add_systemd_prefix:
         ret["channel"] = "systemd-" + ret["channel"]
 
-    # FIXME: This is a hack to work around python/typeshed issue #12919
-    return cast(dict[str, Any], ret)
+    return ret
 
 
 def all_channels() -> list[str]:
     """Get a list of all channels for all pkgrepos."""
-    ret = set()
-    for repo in pkgrepo_paths():
-        ret.add(read_config(repo)["channel"])
+    ret = {read_config(repo)["channel"] for repo in pkgrepo_paths()}
 
-    logging.verbose(f"all_chanels: {ret}")
+    logging.verbose(f"all_channels: {ret}")
     return list(ret)
 
 
@@ -187,12 +186,15 @@ def read_config_channel() -> dict[str, str]:
     )
 
 
-def init() -> None:
+def init(do_shallow_clone: bool = False) -> None:
     if not os.path.exists(pkgrepo_default_path()):
-        clone()
+        clone(do_shallow_clone)
     read_config()
 
 
+DEVELOPMENT_CHANNEL: Final[str] = "edge"
+
+
 def switch_to_channel_branch(channel_new: str) -> bool:
     """Checkout the channel's branch in pmaports.git.
 
@@ -200,38 +202,65 @@ def switch_to_channel_branch(channel_new: str) -> bool:
 
     :returns: True if another branch was checked out, False otherwise
     """
-    # Check current pmaports branch channel
-    channel_current = read_config()["channel"]
-    if channel_current == channel_new:
-        return False
-
     aports = pkgrepo_default_path()
+
     # list current and new branches/channels
     channels_cfg = pmb.helpers.git.parse_channels_cfg(aports)
     branch_new = channels_cfg["channels"][channel_new]["branch_pmaports"]
     branch_current = pmb.helpers.git.rev_parse(aports, extra_args=["--abbrev-ref"])
+
+    # Check current pmaports branch channel
+    channel_current = read_config(aports)["channel"]
+    # If we are on edge and on master but edge calls for main
+    if (
+        channel_current == DEVELOPMENT_CHANNEL
+        and branch_current == "master"
+        and branch_new == "main"
+    ):
+        logging.info("NOTE: master was renamed to main, switching to it")
+        branch_new = "main"
+    elif channel_current == channel_new:
+        return False
+
     if (
         branch_current == "master_staging_systemd"
-        and channel_new == "edge"
+        and channel_new == DEVELOPMENT_CHANNEL
         and pmb.config.is_systemd_selected()
     ):
-        logging.info(
-            "NOTE: systemd enabled and currently on the master_staging_systemd branch, not switching to edge"
-        )
-        return False
+        logging.info("NOTE: master_staging_systemd was merged into master, switching to it")
 
     logging.info(
         f"Currently checked out branch '{branch_current}' of"
         f" pmaports.git is on channel '{channel_current}'."
     )
-    logging.info(f"Switching to branch '{branch_new}' on channel" f" '{channel_new}'...")
+    logging.info(f"Switching to branch '{branch_new}' on channel '{channel_new}'...")
 
     # Make sure we don't have mounts related to the old channel
     pmb.chroot.shutdown()
 
+    # We need to make sure the branch exists in case --shallow-initial-clone previously was used.
+    # Checking whether args.shallow_initial_clone is True would not solve this problem as a user
+    # could use --shallow-initial-clone when cloning for the first time, then call init without the
+    # flag subsequent times without the repository ever ending up "deep cloned". As such, check for
+    # the presence of the branch instead of whether the flag is True.
+    if pmb.helpers.run.user(
+        ["git", "show-ref", "--quiet", "--branches", branch_new], aports, check=False
+    ):
+        logging.info(f"Branch {branch_new} doesn't exist, attempting to fetch it")
+
+        if pmb.helpers.run.user(
+            ["git", "fetch", "--depth=1", "origin", f"{branch_new}:{branch_new}"],
+            aports,
+            RunOutputTypeDefault.INTERACTIVE,
+            check=False,
+        ):
+            raise NonBugError(f"Failed to fetch {branch_new}!")
+
     # Attempt to switch branch (git gives a nice error message, mentioning
     # which files need to be committed/stashed, so just pass it through)
-    if pmb.helpers.run.user(["git", "checkout", branch_new], aports, "interactive", check=False):
+    if pmb.helpers.run.user(
+        ["git", "switch", branch_new], aports, RunOutputTypeDefault.INTERACTIVE, check=False
+    ):
         raise RuntimeError(
             "Failed to switch branch. Go to your pmaports and"
             " fix what git complained about, then try again: "
