@@ -7,7 +7,6 @@ import os
 
 from pmb.types import PmbArgs
 import pmb.helpers.run
-import pmb.helpers.frontend
 import pmb.chroot.initfs
 import pmb.export
 from pmb.core import Chroot, ChrootType
@@ -30,12 +29,11 @@ def frontend(args: PmbArgs) -> None:  # FIXME: ARGS_REFACTOR
         )
 
     # Rebuild the initramfs, just to make sure (see #69)
-    flavor = pmb.helpers.frontend._parse_flavor(config.device, args.autoinstall)
     if args.autoinstall:
-        pmb.chroot.initfs.build(flavor, Chroot(ChrootType.ROOTFS, config.device))
+        pmb.chroot.initfs.build(Chroot(ChrootType.ROOTFS, config.device))
 
     # Do the export, print all files
     logging.info(f"Export symlinks to: {target}")
     if args.odin_flashable_tar:
-        pmb.export.odin(config.device, flavor, target)
+        pmb.export.odin(config.device, target)
     pmb.export.symlinks(target)
