@@ -1,17 +1,74 @@
 # Copyright 2023 Oliver Smith
 # SPDX-License-Identifier: GPL-3.0-or-later
+
+from __future__ import annotations
+
 import copy
+from dataclasses import dataclass
+from enum import Enum
 from pathlib import Path
 from pmb.core.context import get_context
 from pmb.core.arch import Arch
 from pmb.helpers import logging
+from pmb.helpers.exceptions import NonBugError
+import inspect
 import os
 import pmb.config
-import pmb.helpers.other
 import pmb.helpers.devices
 from pmb.meta import Cache
 
 
+class InitfsCompressionFormat(Enum):
+    ZSTD = "zstd"
+    LZ4 = "lz4"
+    LZMA = "lzma"
+    GZIP = "gzip"
+    NONE = "none"
+
+    @staticmethod
+    def from_str(compression_format: str) -> InitfsCompressionFormat:
+        try:
+            return InitfsCompressionFormat(compression_format)
+        except ValueError as exception:
+            raise ValueError(f"Invalid compression format '{compression_format}'") from exception
+
+
+class InitfsCompressionLevel(Enum):
+    DEFAULT = "default"
+    FAST = "fast"
+    BEST = "best"
+
+    @staticmethod
+    def from_str(compression_level: str) -> InitfsCompressionLevel:
+        try:
+            return InitfsCompressionLevel(compression_level)
+        except ValueError as exception:
+            raise ValueError(f"Invalid compression level '{compression_level}'") from exception
+
+
+@dataclass
+class InitfsCompression:
+    format_: InitfsCompressionFormat
+    level: InitfsCompressionLevel | None
+
+    @staticmethod
+    def from_str(initfs_compression: str) -> InitfsCompression | None:
+        segments = initfs_compression.split(":", maxsplit=1)
+
+        try:
+            format_ = InitfsCompressionFormat.from_str(segments[0])
+        except ValueError:
+            # If we can't even figure out the format, the other information is no use.
+            return None
+
+        try:
+            level = InitfsCompressionLevel.from_str(segments[1]) if len(segments) == 2 else None
+        except ValueError:
+            level = None
+
+        return InitfsCompression(format_, level)
+
+
 # FIXME: It feels weird to handle this at parse time.
 # we should instead have the Deviceinfo object store
 # the attributes for all kernels and require the user
@@ -53,7 +110,7 @@ def _parse_kernel_suffix(info: dict[str, str], device: str, kernel: str | None)
     ret = copy.copy(info)
 
     suffix_kernel = kernel.replace("-", "_")
-    for key in Deviceinfo.__annotations__.keys():
+    for key in inspect.get_annotations(Deviceinfo):
         key_kernel = f"{key}_{suffix_kernel}"
         if key_kernel not in ret:
             continue
@@ -67,7 +124,7 @@ def _parse_kernel_suffix(info: dict[str, str], device: str, kernel: str | None)
 
 
 @Cache("device", "kernel")
-def deviceinfo(device: str | None = None, kernel: str | None = None) -> "Deviceinfo":
+def deviceinfo(device: str | None = None, kernel: str | None = None) -> Deviceinfo:
     """
     :param device: defaults to args.device
     :param kernel: defaults to args.kernel
@@ -80,10 +137,9 @@ def deviceinfo(device: str | None = None, kernel: str | None = None) -> "Devicei
 
     path = pmb.helpers.devices.find_path(device, "deviceinfo")
     if not path:
-        raise RuntimeError(
-            "Device '" + device + "' not found. Run 'pmbootstrap init' to"
-            " start a new device port or to choose another device. It may have"
-            " been renamed, see <https://postmarketos.org/renamed>"
+        raise NonBugError(
+            f"Device '{device}' not found. Run 'pmbootstrap init' to start a new device port or to "
+            "choose another device. It may have been renamed, see <https://postmarketos.org/renamed>"
         )
 
     return Deviceinfo(path, kernel)
@@ -119,31 +175,38 @@ class Deviceinfo:
     flash_method: str = ""
     boot_filesystem: str | None = ""
     create_initfs_extra: bool | None = False
+    create_prep_boot: bool | None = False
+    initfs_compression: InitfsCompression = InitfsCompression(InitfsCompressionFormat.GZIP, None)
 
     # flash
     flash_heimdall_partition_kernel: str | None = ""
+    flash_heimdall_partition_vendor_boot: str | None = ""
     flash_heimdall_partition_initfs: str | None = ""
     flash_heimdall_partition_rootfs: str | None = ""
     flash_heimdall_partition_system: str | None = ""  # deprecated
     flash_heimdall_partition_vbmeta: str | None = ""
     flash_heimdall_partition_dtbo: str | None = ""
     flash_fastboot_partition_kernel: str | None = ""
+    flash_fastboot_partition_vendor_boot: str | None = ""
     flash_fastboot_partition_rootfs: str | None = ""
     flash_fastboot_partition_system: str | None = ""  # deprecated
     flash_fastboot_partition_vbmeta: str | None = ""
     flash_fastboot_partition_dtbo: str | None = ""
     flash_rk_partition_kernel: str | None = ""
+    flash_rk_partition_vendor_boot: str | None = ""
     flash_rk_partition_rootfs: str | None = ""
     flash_rk_partition_system: str | None = ""  # deprecated
     flash_mtkclient_partition_kernel: str | None = ""
+    flash_mtkclient_partition_vendor_boot: str | None = ""
     flash_mtkclient_partition_rootfs: str | None = ""
     flash_mtkclient_partition_vbmeta: str | None = ""
     flash_mtkclient_partition_dtbo: str | None = ""
     generate_legacy_uboot_initfs: str | None = ""
     kernel_cmdline: str | None = ""
     generate_bootimg: str | None = ""
-    header_version: str | None = ""
+    header_version: int | None = None
     bootimg_qcdt: str | None = ""
+    bootimg_qcdt_type: str | None = ""
     bootimg_mtk_mkimage: str | None = ""  # deprecated
     bootimg_mtk_label_kernel: str | None = ""
     bootimg_mtk_label_ramdisk: str | None = ""
@@ -257,7 +320,7 @@ class Deviceinfo:
                 f" and try again: {path}"
             )
 
-    def __init__(self, path: Path, kernel: str | None = None):
+    def __init__(self, path: Path, kernel: str | None = None) -> None:
         ret = {}
         with open(path) as handle:
             for line in handle:
@@ -277,12 +340,17 @@ class Deviceinfo:
             # FIXME: something to turn on and fix in the future
             # if key not in Deviceinfo.__annotations__.keys():
             #     logging.warning(f"deviceinfo: {key} is not a known attribute")
-            if key == "arch":
-                setattr(self, key, Arch.from_str(value))
-            elif key == "gpu_accelerated":  # deprecated
-                setattr(self, "drm", value)
-            else:
-                setattr(self, key, value)
+            match key:
+                case "arch":
+                    setattr(self, key, Arch.from_str(value))
+                case "gpu_accelerated":  # deprecated
+                    setattr(self, "drm", value)
+                case "header_version":
+                    setattr(self, key, int(value))
+                case "initfs_compression":
+                    setattr(self, key, InitfsCompression.from_str(value))
+                case _:
+                    setattr(self, key, value)
 
         if not self.flash_method:
             self.flash_method = "none"
